@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -31,7 +31,7 @@
  ****************************************************************************/
 #include "form.priv.h"
 
-MODULE_ID("$Id: fld_opts.c,v 1.6 1999/05/16 17:19:06 juergen Exp $")
+MODULE_ID("$Id: fld_opts.c,v 1.7 2000/12/10 02:09:38 tom Exp $")
 
 /*----------------------------------------------------------------------------
   Field-Options manipulation routines
@@ -49,7 +49,8 @@ MODULE_ID("$Id: fld_opts.c,v 1.6 1999/05/16 17:19:06 juergen Exp $")
 |                    E_BAD_ARGUMENT  - invalid options
 |                    E_SYSTEM_ERROR  - system error
 +--------------------------------------------------------------------------*/
-int set_field_opts(FIELD * field, Field_Options opts)
+NCURSES_EXPORT(int)
+set_field_opts (FIELD * field, Field_Options opts)
 {
   int res = E_BAD_ARGUMENT;
   opts &= ALL_FIELD_OPTS;
@@ -66,7 +67,8 @@ int set_field_opts(FIELD * field, Field_Options opts)
 |
 |   Return Values :  The options.
 +--------------------------------------------------------------------------*/
-Field_Options field_opts(const FIELD * field)
+NCURSES_EXPORT(Field_Options)
+field_opts (const FIELD * field)
 {
   return ALL_FIELD_OPTS & Normalize_Field( field )->opts;
 }
@@ -83,7 +85,8 @@ Field_Options field_opts(const FIELD * field)
 |                    E_BAD_ARGUMENT  - invalid options
 |                    E_SYSTEM_ERROR  - system error
 +--------------------------------------------------------------------------*/
-int field_opts_on(FIELD * field, Field_Options opts)
+NCURSES_EXPORT(int)
+field_opts_on (FIELD * field, Field_Options opts)
 {
   int res = E_BAD_ARGUMENT;
 
@@ -108,7 +111,8 @@ int field_opts_on(FIELD * field, Field_Options opts)
 |                    E_BAD_ARGUMENT  - invalid options
 |                    E_SYSTEM_ERROR  - system error
 +--------------------------------------------------------------------------*/
-int field_opts_off(FIELD  * field, Field_Options opts)
+NCURSES_EXPORT(int)
+field_opts_off (FIELD  * field, Field_Options opts)
 {
   int res = E_BAD_ARGUMENT;
 
