@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -31,7 +31,7 @@
  ****************************************************************************/
 #include "form.priv.h"
 
-MODULE_ID("$Id: fld_pad.c,v 1.4 1999/05/16 17:38:58 juergen Exp $")
+MODULE_ID("$Id: fld_pad.c,v 1.5 2000/12/10 02:09:38 tom Exp $")
 
 /*---------------------------------------------------------------------------
 |   Facility      :  libnform  
@@ -44,7 +44,8 @@ MODULE_ID("$Id: fld_pad.c,v 1.4 1999/05/16 17:38:58 juergen Exp $")
 |                    E_BAD_ARGUMENT - invalid field pointer or pad character
 |                    E_SYSTEM_ERROR - system error
 +--------------------------------------------------------------------------*/
-int set_field_pad(FIELD  * field, int ch)
+NCURSES_EXPORT(int)
+set_field_pad (FIELD  * field, int ch)
 {
   int res = E_BAD_ARGUMENT;
 
@@ -70,7 +71,8 @@ int set_field_pad(FIELD  * field, int ch)
 |
 |   Return Values :  The pad character.
 +--------------------------------------------------------------------------*/
-int field_pad(const FIELD * field)
+NCURSES_EXPORT(int)
+field_pad (const FIELD * field)
 {
   return Normalize_Field( field )->pad;
 }
