@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -32,7 +32,7 @@
 
 #include "form.priv.h"
 
-MODULE_ID("$Id: fld_page.c,v 1.4 1999/05/16 17:19:37 juergen Exp $")
+MODULE_ID("$Id: fld_page.c,v 1.5 2000/12/10 02:09:38 tom Exp $")
 
 /*---------------------------------------------------------------------------
 |   Facility      :  libnform  
@@ -44,7 +44,8 @@ MODULE_ID("$Id: fld_page.c,v 1.4 1999/05/16 17:19:37 juergen Exp $")
 |   Return Values :  E_OK         - success
 |                    E_CONNECTED  - field is connected
 +--------------------------------------------------------------------------*/
-int set_new_page(FIELD * field, bool new_page_flag)
+NCURSES_EXPORT(int)
+set_new_page (FIELD * field, bool new_page_flag)
 {
   Normalize_Field(field);
   if (field->form) 
@@ -68,7 +69,8 @@ int set_new_page(FIELD * field, bool new_page_flag)
 |   Return Values :  TRUE  - field starts a new page
 |                    FALSE - field doesn't start a new page
 +--------------------------------------------------------------------------*/
-bool new_page(const FIELD * field)
+NCURSES_EXPORT(bool)
+new_page (const FIELD * field)
 {
   return (Normalize_Field(field)->status & _NEWPAGE)  ? TRUE : FALSE;
 }
