@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -32,7 +32,7 @@
 
 #include "form.priv.h"
 
-MODULE_ID("$Id: fld_stat.c,v 1.6 1999/05/16 17:19:48 juergen Exp $")
+MODULE_ID("$Id: fld_stat.c,v 1.7 2000/12/10 02:09:38 tom Exp $")
 
 /*---------------------------------------------------------------------------
 |   Facility      :  libnform  
@@ -43,7 +43,8 @@ MODULE_ID("$Id: fld_stat.c,v 1.6 1999/05/16 17:19:48 juergen Exp $")
 |
 |   Return Values :  E_OK            - success
 +--------------------------------------------------------------------------*/
-int set_field_status(FIELD * field, bool status)
+NCURSES_EXPORT(int)
+set_field_status (FIELD * field, bool status)
 {
   Normalize_Field( field );
 
@@ -65,7 +66,8 @@ int set_field_status(FIELD * field, bool status)
 |   Return Values :  TRUE  - buffer has been changed
 |                    FALSE - buffer has not been changed
 +--------------------------------------------------------------------------*/
-bool field_status(const FIELD * field)
+NCURSES_EXPORT(bool)
+field_status (const FIELD * field)
 {
   return ((Normalize_Field(field)->status & _CHANGED) ? TRUE : FALSE);
 }
