@@ -13,7 +13,7 @@
 
 #include "form.priv.h"
 
-MODULE_ID("$Id: fty_alnum.c,v 1.9 1999/05/16 17:22:49 juergen Exp $")
+MODULE_ID("$Id: fty_alnum.c,v 1.10 2000/12/09 23:46:12 tom Exp $")
 
 typedef struct {
   int width;
@@ -132,6 +132,6 @@ static FIELDTYPE typeALNUM = {
   NULL
 };
 
-FIELDTYPE* TYPE_ALNUM = &typeALNUM;
+NCURSES_EXPORT_VAR(FIELDTYPE*) TYPE_ALNUM = &typeALNUM;
 
 /* fty_alnum.c ends here */
