@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,1999,2000 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998,1999,2000,2001 Free Software Foundation, Inc.         *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -40,11 +40,12 @@
 
 #include <curses.priv.h>
 
-MODULE_ID("$Id: lib_getch.c,v 1.50 2000/10/09 23:53:57 Ilya.Zakharevich Exp $")
+MODULE_ID("$Id: lib_getch.c,v 1.58 2001/12/18 23:27:45 tom Exp $")
 
 #include <fifo_defs.h>
 
-int ESCDELAY = 1000;		/* max interval betw. chars in funkeys, in millisecs */
+NCURSES_EXPORT_VAR(int)
+ESCDELAY = 1000;		/* max interval betw. chars in funkeys, in millisecs */
 
 static inline int
 fifo_peek(void)
@@ -61,7 +62,7 @@ fifo_pull(void)
 {
     int ch;
     ch = SP->_fifo[head];
-    TR(TRACE_IEVENT, ("pulling %d from %d", ch, head));
+    TR(TRACE_IEVENT, ("pulling %s from %d", _tracechar(ch), head));
 
     if (peek == head) {
 	h_inc();
@@ -80,7 +81,7 @@ static inline int
 fifo_push(void)
 {
     int n;
-    unsigned int ch;
+    int ch;
 
     if (tail == -1)
 	return ERR;
@@ -101,7 +102,7 @@ fifo_push(void)
     {
 	unsigned char c2 = 0;
 	n = read(SP->_ifd, &c2, 1);
-	ch = c2 & 0xff;
+	ch = c2;
     }
 
 #ifdef HIDE_EINTR
@@ -129,7 +130,7 @@ fifo_push(void)
     if (head == -1)
 	head = peek = tail;
     t_inc();
-    TR(TRACE_IEVENT, ("pushed %#x at %d", ch, tail));
+    TR(TRACE_IEVENT, ("pushed %s at %d", _tracechar(ch), tail));
 #ifdef TRACE
     if (_nc_tracing & TRACE_IEVENT)
 	_nc_fifo_dump();
@@ -153,7 +154,7 @@ static int kgetch(WINDOW *);
 	(is_wintouched(win) || (win->_flags & _HASMOVED)) \
 	&& !(win->_flags & _ISPAD))
 
-int
+NCURSES_EXPORT(int)
 wgetch(WINDOW *win)
 {
     int ch;
@@ -168,7 +169,7 @@ wgetch(WINDOW *win)
 	    wrefresh(win);
 
 	ch = fifo_pull();
-	T(("wgetch returning (pre-cooked): %#x = %s", ch, _trace_key(ch)));
+	T(("wgetch returning (pre-cooked): %s", _tracechar(ch)));
 	returnCode(ch);
     }
 
@@ -192,6 +193,9 @@ wgetch(WINDOW *win)
 	returnCode(fifo_pull());
     }
 
+    if (win->_use_keypad != SP->_keypad_on)
+	_nc_keypad(win->_use_keypad);
+
     if (wgetch_should_refresh(win))
 	wrefresh(win);
 
@@ -233,10 +237,12 @@ wgetch(WINDOW *win)
 		if (SP->_mouse_inline(SP))
 		    break;
 	    }
+	    if (SP->_maxclick < 0)
+		break;
 	} while
 	    (ch == KEY_MOUSE
-	    && (_nc_timed_wait(3, SP->_maxclick, (int *) 0)
-		|| !SP->_mouse_parse(runcount)));
+	     && (_nc_timed_wait(3, SP->_maxclick, (int *) 0)
+		 || !SP->_mouse_parse(runcount)));
 	if (runcount > 0 && ch != KEY_MOUSE) {
 	    /* mouse event sequence ended by keystroke, push it */
 	    ungetch(ch);
@@ -255,7 +261,7 @@ wgetch(WINDOW *win)
 	    /* resizeterm can push KEY_RESIZE */
 	    if (cooked_key_in_fifo()) {
 		ch = fifo_pull();
-		T(("wgetch returning (pre-cooked): %#x = %s", ch, _trace_key(ch)));
+		T(("wgetch returning (pre-cooked): %s", _tracechar(ch)));
 		returnCode(ch);
 	    }
 	}
@@ -303,7 +309,7 @@ wgetch(WINDOW *win)
 	if (!SP->_use_meta)
 	    ch &= 0x7f;
 
-    T(("wgetch returning : %#x = %s", ch, _trace_key(ch)));
+    T(("wgetch returning : %s", _tracechar(ch)));
 
     returnCode(ch);
 }
@@ -349,19 +355,16 @@ kgetch(WINDOW *win GCC_UNUSED)
 	    return ch;
 	}
 
-	TR(TRACE_IEVENT, ("ch: %s", _trace_key((unsigned char) ch)));
+	TR(TRACE_IEVENT, ("ch: %s", _tracechar((unsigned char) ch)));
 	while ((ptr != NULL) && (ptr->ch != (unsigned char) ch))
 	    ptr = ptr->sibling;
-#ifdef TRACE
+
 	if (ptr == NULL) {
 	    TR(TRACE_IEVENT, ("ptr is null"));
-	} else
-	    TR(TRACE_IEVENT, ("ptr=%p, ch=%d, value=%d",
-		    ptr, ptr->ch, ptr->value));
-#endif /* TRACE */
-
-	if (ptr == NULL)
 	    break;
+	}
+	TR(TRACE_IEVENT, ("ptr=%p, ch=%d, value=%d",
+			  ptr, ptr->ch, ptr->value));
 
 	if (ptr->value != 0) {	/* sequence terminated */
 	    TR(TRACE_IEVENT, ("end of sequence"));
