@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,1999,2000 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998,1999,2000,2002 Free Software Foundation, Inc.         *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -84,7 +84,7 @@
 #endif
 #endif
 
-MODULE_ID("$Id: lib_mouse.c,v 1.55 2000/10/10 00:07:28 Ilya.Zakharevich Exp $")
+MODULE_ID("$Id: lib_mouse.c,v 1.58 2002/01/12 22:38:07 tom Exp $")
 
 #define MY_TRACE TRACE_ICALLS|TRACE_IEVENT
 
@@ -655,7 +655,7 @@ _nc_mouse_parse(int runcount)
     /* first pass; merge press/release pairs */
     do {
 	merge = FALSE;
-	for (ep = runp; next = NEXT(ep), next != eventp; ep = next) {
+	for (ep = runp; (next = NEXT(ep)) != eventp; ep = next) {
 	    if (ep->x == next->x && ep->y == next->y
 		&& (ep->bstate & (BUTTON1_PRESSED | BUTTON2_PRESSED | BUTTON3_PRESSED))
 		&& (!(ep->bstate & BUTTON1_PRESSED)
@@ -720,7 +720,7 @@ _nc_mouse_parse(int runcount)
 	MEVENT *follower;
 
 	merge = FALSE;
-	for (ep = runp; next = NEXT(ep), next != eventp; ep = next)
+	for (ep = runp; (next = NEXT(ep)) != eventp; ep = next)
 	    if (ep->id != INVALID_EVENT) {
 		if (next->id != INVALID_EVENT)
 		    continue;
@@ -863,7 +863,7 @@ _nc_mouse_resume(SCREEN * sp GCC_UNUSED)
  *
  **************************************************************************/
 
-int
+NCURSES_EXPORT(int)
 getmouse(MEVENT * aevent)
 /* grab a copy of the current mouse event */
 {
@@ -886,7 +886,7 @@ getmouse(MEVENT * aevent)
     returnCode(ERR);
 }
 
-int
+NCURSES_EXPORT(int)
 ungetmouse(MEVENT * aevent)
 /* enqueue a synthesized mouse event to be seen by the next wgetch() */
 {
@@ -900,7 +900,7 @@ ungetmouse(MEVENT * aevent)
     return ungetch(KEY_MOUSE);
 }
 
-mmask_t
+NCURSES_EXPORT(mmask_t)
 mousemask(mmask_t newmask, mmask_t * oldmask)
 /* set the mouse event mask */
 {
@@ -933,7 +933,7 @@ mousemask(mmask_t newmask, mmask_t * oldmask)
     returnCode(result);
 }
 
-bool
+NCURSES_EXPORT(bool)
 wenclose(const WINDOW *win, int y, int x)
 /* check to see if given window encloses given screen location */
 {
@@ -947,7 +947,7 @@ wenclose(const WINDOW *win, int y, int x)
     return FALSE;
 }
 
-int
+NCURSES_EXPORT(int)
 mouseinterval(int maxclick)
 /* set the maximum mouse interval within which to recognize a click */
 {
@@ -966,14 +966,15 @@ mouseinterval(int maxclick)
 
 /* This may be used by other routines to ask for the existence of mouse
    support */
-int
+NCURSES_EXPORT(int)
 _nc_has_mouse(void)
 {
     return (mousetype == M_NONE ? 0 : 1);
 }
 
-bool
-wmouse_trafo(const WINDOW *win, int *pY, int *pX, bool to_screen)
+NCURSES_EXPORT(bool)
+wmouse_trafo
+(const WINDOW *win, int *pY, int *pX, bool to_screen)
 {
     bool result = FALSE;
 
