@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,1999,2000 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998,1999,2000,2001 Free Software Foundation, Inc.         *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -43,9 +43,9 @@
 #include <term.h>		/* cur_term */
 #include <tic.h>
 
-MODULE_ID("$Id: lib_set_term.c,v 1.58 2000/10/04 22:05:48 tom Exp $")
+MODULE_ID("$Id: lib_set_term.c,v 1.63 2001/09/01 21:42:14 tom Exp $")
 
-SCREEN *
+NCURSES_EXPORT(SCREEN *)
 set_term(SCREEN * screenp)
 {
     SCREEN *oldSP;
@@ -80,7 +80,7 @@ _nc_free_keytry(struct tries *kt)
 /*
  * Free the storage associated with the given SCREEN sp.
  */
-void
+NCURSES_EXPORT(void)
 delscreen(SCREEN * sp)
 {
     SCREEN **scan = &_nc_screen_chain;
@@ -95,9 +95,9 @@ delscreen(SCREEN * sp)
 	scan = &(*scan)->_next_screen;
     }
 
-    _nc_freewin(sp->_curscr);
-    _nc_freewin(sp->_newscr);
-    _nc_freewin(sp->_stdscr);
+    (void) _nc_freewin(sp->_curscr);
+    (void) _nc_freewin(sp->_newscr);
+    (void) _nc_freewin(sp->_stdscr);
     _nc_free_keytry(sp->_keytry);
     _nc_free_keytry(sp->_key_ok);
 
@@ -191,8 +191,9 @@ extract_fgbg(char *src, int *result)
 }
 #endif
 
-int
-_nc_setupscreen(short slines, short const scolumns, FILE * output)
+NCURSES_EXPORT(int)
+_nc_setupscreen
+(short slines, short const scolumns, FILE * output)
 /* OS-independent screen initializations */
 {
     int bottom_stolen = 0;
@@ -224,7 +225,7 @@ _nc_setupscreen(short slines, short const scolumns, FILE * output)
 #if NCURSES_NO_PADDING
     SP->_no_padding = getenv("NCURSES_NO_PADDING") != 0;
     TR(TRACE_CHARPUT | TRACE_MOVE, ("padding will%s be used",
-	    SP->_no_padding ? " not" : ""));
+				    SP->_no_padding ? " not" : ""));
 #endif
 
 #if NCURSES_EXT_FUNCS
@@ -267,8 +268,13 @@ _nc_setupscreen(short slines, short const scolumns, FILE * output)
      */
     if (getenv("COLORFGBG") != 0) {
 	char *p = getenv("COLORFGBG");
+	TR(TRACE_CHARPUT | TRACE_MOVE, ("decoding COLORFGBG %s", p));
 	p = extract_fgbg(p, &(SP->_default_fg));
 	p = extract_fgbg(p, &(SP->_default_bg));
+	if (*p)			/* assume rxvt was compiled with xpm support */
+	    p = extract_fgbg(p, &(SP->_default_bg));
+	TR(TRACE_CHARPUT | TRACE_MOVE, ("decoded fg=%d, bg=%d",
+					SP->_default_fg, SP->_default_bg));
     }
 #endif
 #endif /* NCURSES_EXT_FUNCS */
@@ -295,12 +301,12 @@ _nc_setupscreen(short slines, short const scolumns, FILE * output)
     if (magic_cookie_glitch > 0) {
 
 	SP->_xmc_triggers = termattrs() & (
-	    A_ALTCHARSET |
-	    A_BLINK |
-	    A_BOLD |
-	    A_REVERSE |
-	    A_STANDOUT |
-	    A_UNDERLINE
+					      A_ALTCHARSET |
+					      A_BLINK |
+					      A_BOLD |
+					      A_REVERSE |
+					      A_STANDOUT |
+					      A_UNDERLINE
 	    );
 	SP->_xmc_suppress = SP->_xmc_triggers & (chtype) ~ (A_BOLD);
 
@@ -346,6 +352,9 @@ _nc_setupscreen(short slines, short const scolumns, FILE * output)
     newscr->_clear = TRUE;
     curscr->_clear = FALSE;
 
+    def_shell_mode();
+    def_prog_mode();
+
     for (i = 0, rsp = rippedoff; rsp->line && (i < N_RIPS); rsp++, i++) {
 	if (rsp->hook) {
 	    WINDOW *w;
@@ -381,16 +390,13 @@ _nc_setupscreen(short slines, short const scolumns, FILE * output)
 	return ERR;
     SP->_stdscr = stdscr;
 
-    def_shell_mode();
-    def_prog_mode();
-
     return OK;
 }
 
 /* The internal implementation interprets line as the number of
    lines to rip off from the top or bottom.
    */
-int
+NCURSES_EXPORT(int)
 _nc_ripoffline(int line, int (*init) (WINDOW *, int))
 {
     if (line == 0)
@@ -407,7 +413,7 @@ _nc_ripoffline(int line, int (*init) (WINDOW *, int))
     return (OK);
 }
 
-int
+NCURSES_EXPORT(int)
 ripoffline(int line, int (*init) (WINDOW *, int))
 {
     T((T_CALLED("ripoffline(%d,%p)"), line, init));
