@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,1999,2000 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998,1999,2000,2001 Free Software Foundation, Inc.         *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -32,7 +32,7 @@
 
 #include <curses.priv.h>
 
-MODULE_ID("$Id: wresize.c,v 1.16 2000/03/05 00:14:35 tom Exp $")
+MODULE_ID("$Id: wresize.c,v 1.20 2001/12/19 01:07:07 tom Exp $")
 
 /*
  * Reallocate a curses WINDOW struct to either shrink or grow to the specified
@@ -42,23 +42,23 @@ MODULE_ID("$Id: wresize.c,v 1.16 2000/03/05 00:14:35 tom Exp $")
 
 #define DOALLOC(p,t,n)  typeRealloc(t, n, p)
 #define	ld_ALLOC(p,n)	DOALLOC(p,struct ldat,n)
-#define	c_ALLOC(p,n)	DOALLOC(p,chtype,n)
+#define	c_ALLOC(p,n)	DOALLOC(p,NCURSES_CH_T,n)
 
-int
+NCURSES_EXPORT(int)
 wresize(WINDOW *win, int ToLines, int ToCols)
 {
     register int row;
     int size_x, size_y;
     struct ldat *pline;
-    chtype blank;
+    NCURSES_CH_T blank;
 
 #ifdef TRACE
     T((T_CALLED("wresize(%p,%d,%d)"), win, ToLines, ToCols));
     if (win) {
 	TR(TRACE_UPDATE, ("...beg (%d, %d), max(%d,%d), reg(%d,%d)",
-		win->_begy, win->_begx,
-		win->_maxy, win->_maxx,
-		win->_regtop, win->_regbottom));
+			  win->_begy, win->_begx,
+			  win->_maxy, win->_maxx,
+			  win->_regtop, win->_regbottom));
 	if (_nc_tracing & TRACE_UPDATE)
 	    _tracedump("...before", win);
     }
@@ -117,9 +117,9 @@ wresize(WINDOW *win, int ToLines, int ToCols)
     /*
      * Adjust the width of the columns:
      */
-    blank = _nc_background(win);
+    blank = win->_nc_bkgd;
     for (row = 0; row <= ToLines; row++) {
-	chtype *s = win->_line[row].text;
+	NCURSES_CH_T *s = win->_line[row].text;
 	int begin = (s == 0) ? 0 : size_x + 1;
 	int end = ToCols;
 
@@ -169,9 +169,9 @@ wresize(WINDOW *win, int ToLines, int ToCols)
 
 #ifdef TRACE
     TR(TRACE_UPDATE, ("...beg (%d, %d), max(%d,%d), reg(%d,%d)",
-	    win->_begy, win->_begx,
-	    win->_maxy, win->_maxx,
-	    win->_regtop, win->_regbottom));
+		      win->_begy, win->_begx,
+		      win->_maxy, win->_maxx,
+		      win->_regtop, win->_regbottom));
     if (_nc_tracing & TRACE_UPDATE)
 	_tracedump("...after:", win);
 #endif
