@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,1999,2000 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998,1999,2000,2001,2002 Free Software Foundation, Inc.    *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,6 +29,7 @@
 /****************************************************************************
  *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1992,1995               *
  *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
+ *     and: Thomas Dickey 1995-2001                                         *
  ****************************************************************************/
 
 /*
@@ -37,17 +38,17 @@
 **	The routine _nc_signal_handler().
 **
 */
+#include <signal.h>
 
 #include <curses.priv.h>
 
-#include <signal.h>
 #include <SigAction.h>
 
 #if SVR4_ACTION && !defined(_POSIX_SOURCE)
 #define _POSIX_SOURCE
 #endif
 
-MODULE_ID("$Id: lib_tstp.c,v 1.22 2000/09/02 18:33:17 tom Exp $")
+MODULE_ID("$Id: lib_tstp.c,v 1.29 2002/01/12 17:40:49 tom Exp $")
 
 #if defined(SIGTSTP) && (HAVE_SIGACTION || HAVE_SIGVEC)
 #define USE_SIGTSTP 1
@@ -55,6 +56,41 @@ MODULE_ID("$Id: lib_tstp.c,v 1.22 2000/09/02 18:33:17 tom Exp $")
 #define USE_SIGTSTP 0
 #endif
 
+#ifdef TRACE
+static const char *
+signal_name(int sig)
+{
+    switch (sig) {
+    case SIGALRM:
+	return "SIGALRM";
+#ifdef SIGCONT
+    case SIGCONT:
+	return "SIGCONT";
+#endif
+    case SIGINT:
+	return "SIGINT";
+    case SIGQUIT:
+	return "SIGQUIT";
+    case SIGTERM:
+	return "SIGTERM";
+#ifdef SIGTSTP
+    case SIGTSTP:
+	return "SIGTSTP";
+#endif
+#ifdef SIGTTOU
+    case SIGTTOU:
+	return "SIGTTOU";
+#endif
+#ifdef SIGWINCH
+    case SIGWINCH:
+	return "SIGWINCH";
+#endif
+    default:
+	return "unknown signal";
+    }
+}
+#endif
+
 /*
  * Note: This code is fragile!  Its problem is that different OSs
  * handle restart of system calls interrupted by signals differently.
@@ -215,7 +251,7 @@ cleanup(int sig)
 	sigemptyset(&act.sa_mask);
 	act.sa_flags = 0;
 	act.sa_handler = SIG_IGN;
-	if (sigaction(sig, &act, (sigaction_t *) 0) == 0)
+	if (sigaction(sig, &act, NULL) == 0)
 #else
 	if (signal(sig, SIG_IGN) != SIG_ERR)
 #endif
@@ -255,43 +291,55 @@ sigwinch(int sig GCC_UNUSED)
  * If the given signal is still in its default state, set it to the given
  * handler.
  */
-#if HAVE_SIGACTION || HAVE_SIGVEC
 static int
-CatchIfDefault(int sig, sigaction_t * act)
+CatchIfDefault(int sig, RETSIGTYPE(*handler) (int))
 {
+    int result;
+#if HAVE_SIGACTION || HAVE_SIGVEC
     sigaction_t old_act;
+    sigaction_t new_act;
 
-    if (sigaction(sig, (sigaction_t *) 0, &old_act) == 0
+    memset(&new_act, 0, sizeof(new_act));
+    sigemptyset(&new_act.sa_mask);
+#ifdef SA_RESTART
+    if (sig != SIGWINCH)
+	new_act.sa_flags |= SA_RESTART;
+#endif /* SA_RESTART */
+    new_act.sa_handler = handler;
+
+    if (sigaction(sig, NULL, &old_act) == 0
 	&& (old_act.sa_handler == SIG_DFL
+	    || old_act.sa_handler == handler
 #if USE_SIGWINCH
 	    || (sig == SIGWINCH && old_act.sa_handler == SIG_IGN)
 #endif
 	)) {
-	(void) sigaction(sig, act, (sigaction_t *) 0);
-	return TRUE;
+	(void) sigaction(sig, &new_act, NULL);
+	result = TRUE;
+    } else {
+	result = FALSE;
     }
-    return FALSE;
-}
-#else
-static int
-CatchIfDefault(int sig, RETSIGTYPE(*handler) (int))
-{
-    void (*ohandler) (int);
+#else /* !HAVE_SIGACTION */
+    RETSIGTYPE(*ohandler) (int);
 
     ohandler = signal(sig, SIG_IGN);
     if (ohandler == SIG_DFL
+	|| ohandler == handler
 #if USE_SIGWINCH
 	|| (sig == SIGWINCH && ohandler == SIG_IGN)
 #endif
 	) {
 	signal(sig, handler);
-	return TRUE;
+	result = TRUE;
     } else {
 	signal(sig, ohandler);
-	return FALSE;
+	result = FALSE;
     }
-}
 #endif
+    T(("CatchIfDefault - will %scatch %s",
+       result ? "" : "not ", signal_name(sig)));
+    return result;
+}
 
 /*
  * This is invoked once at the beginning (e.g., from 'initscr()'), to
@@ -304,63 +352,48 @@ CatchIfDefault(int sig, RETSIGTYPE(*handler) (int))
  * The XSI document implies that we shouldn't keep the SIGTSTP handler if
  * the caller later changes its mind, but that doesn't seem correct.
  */
-void
+NCURSES_EXPORT(void)
 _nc_signal_handler(bool enable)
 {
+    static bool initialized = FALSE;
+
+    T((T_CALLED("_nc_signal_handler(%d)"), enable));
 #if USE_SIGTSTP			/* Xenix 2.x doesn't have SIGTSTP, for example */
-    static sigaction_t act, oact;
-    static int ignore;
+    {
+	static bool ignore_tstp = FALSE;
 
-    if (!ignore) {
-	if (!enable) {
-	    act.sa_handler = SIG_IGN;
-	    sigaction(SIGTSTP, &act, &oact);
-	} else if (act.sa_handler) {
-	    sigaction(SIGTSTP, &oact, NULL);
-	} else {		/*initialize */
-	    sigemptyset(&act.sa_mask);
-	    act.sa_flags = 0;
-#if USE_SIGWINCH
-	    act.sa_handler = sigwinch;
-	    CatchIfDefault(SIGWINCH, &act);
-#endif
+	if (!ignore_tstp) {
+	    static sigaction_t act, oact;
 
+	    if (!enable) {
+		act.sa_handler = SIG_IGN;
+		sigaction(SIGTSTP, &act, &oact);
+	    } else if (act.sa_handler != SIG_DFL) {
+		sigaction(SIGTSTP, &oact, NULL);
+	    } else if (sigaction(SIGTSTP, NULL, &oact) == 0
+		       && (oact.sa_handler == SIG_DFL)) {
+		sigemptyset(&act.sa_mask);
 #ifdef SA_RESTART
-	    act.sa_flags |= SA_RESTART;
+		act.sa_flags |= SA_RESTART;
 #endif /* SA_RESTART */
-	    act.sa_handler = cleanup;
-	    CatchIfDefault(SIGINT, &act);
-	    CatchIfDefault(SIGTERM, &act);
-
-	    act.sa_handler = tstp;
-	    if (!CatchIfDefault(SIGTSTP, &act))
-		ignore = TRUE;
+		act.sa_handler = tstp;
+		(void) sigaction(SIGTSTP, &act, NULL);
+	    } else {
+		ignore_tstp = TRUE;
+	    }
 	}
     }
-#else /* !USE_SIGTSTP */
-    if (enable) {
-#if HAVE_SIGACTION || HAVE_SIGVEC
-	static sigaction_t act;
-	sigemptyset(&act.sa_mask);
-#if USE_SIGWINCH
-	act.sa_handler = sigwinch;
-	CatchIfDefault(SIGWINCH, &act);
-#endif
-#ifdef SA_RESTART
-	act.sa_flags |= SA_RESTART;
-#endif /* SA_RESTART */
-	act.sa_handler = cleanup;
-	CatchIfDefault(SIGINT, &act);
-	CatchIfDefault(SIGTERM, &act);
-
-#else /* !(HAVE_SIGACTION || HAVE_SIGVEC) */
-
-	CatchIfDefault(SIGINT, cleanup);
-	CatchIfDefault(SIGTERM, cleanup);
-#if USE_SIGWINCH
-	CatchIfDefault(SIGWINCH, sigwinch);
-#endif
-#endif /* !(HAVE_SIGACTION || HAVE_SIGVEC) */
-    }
 #endif /* !USE_SIGTSTP */
+
+    if (!initialized) {
+	if (enable) {
+	    CatchIfDefault(SIGINT, cleanup);
+	    CatchIfDefault(SIGTERM, cleanup);
+#if USE_SIGWINCH
+	    CatchIfDefault(SIGWINCH, sigwinch);
+#endif
+	    initialized = TRUE;
+	}
+    }
+    returnVoid;
 }
