@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,1999,2000 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998,1999,2000,2001 Free Software Foundation, Inc.         *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -59,7 +59,7 @@
 # endif
 #endif
 
-MODULE_ID("$Id: lib_twait.c,v 1.39 2000/08/26 19:34:15 tom Exp $")
+MODULE_ID("$Id: lib_twait.c,v 1.43 2001/10/14 00:30:50 tom Exp $")
 
 static long
 _nc_gettime(bool first)
@@ -85,7 +85,7 @@ _nc_gettime(bool first)
     }
     res = (t1 - t0) * 1000;
 #endif
-    T(("%s time: %ld msec", first ? "get" : "elapsed", res));
+    TR(TRACE_IEVENT, ("%s time: %ld msec", first ? "get" : "elapsed", res));
     return res;
 }
 
@@ -102,7 +102,7 @@ _nc_gettime(bool first)
  * If the milliseconds given are -1, the wait blocks until activity on the
  * descriptors.
  */
-int
+NCURSES_EXPORT(int)
 _nc_timed_wait(int mode, int milliseconds, int *timeleft)
 {
     int fd;
@@ -119,7 +119,10 @@ _nc_timed_wait(int mode, int milliseconds, int *timeleft)
 
     long starttime, returntime;
 
-    T(("start twait: %d milliseconds, mode: %d", milliseconds, mode));
+    if (milliseconds < 0)
+	milliseconds = 0;
+    TR(TRACE_IEVENT, ("start twait: %d milliseconds, mode: %d",
+		      milliseconds, mode));
 
 #if PRECISE_GETTIME
   retry:
@@ -227,8 +230,8 @@ _nc_timed_wait(int mode, int milliseconds, int *timeleft)
     if (timeleft)
 	*timeleft = milliseconds;
 
-    T(("end twait: returned %d (%d), remaining time %d msec",
-       result, errno, milliseconds));
+    TR(TRACE_IEVENT, ("end twait: returned %d (%d), remaining time %d msec",
+		      result, errno, milliseconds));
 
     /*
      * Both 'poll()' and 'select()' return the number of file descriptors
