@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,1999,2000 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998,1999,2000,2001 Free Software Foundation, Inc.         *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -39,18 +39,18 @@ DESCRIPTION
 AUTHOR
    Author: Eric S. Raymond <esr@snark.thyrsus.com> 1993
 
-$Id: ncurses.c,v 1.130 2000/07/07 11:05:16 tom Exp $
+$Id: ncurses.c,v 1.150 2002/01/12 23:49:26 tom Exp $
 
 ***************************************************************************/
 
-#include <test.priv.h>
-
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
 #include <assert.h>
 #include <signal.h>
 
+#include <test.priv.h>
+
 #if HAVE_LOCALE_H
 #include <locale.h>
 #endif
@@ -92,7 +92,7 @@ static int save_trace = TRACE_ORDINARY | TRACE_CALLS;
 extern int _nc_tracing;
 #endif
 
-#if !HAVE_NAPMS
+#if !defined(HAVE_NAPMS)
 #define HAVE_NAPMS 1
 #endif
 
@@ -119,6 +119,9 @@ extern int _nc_tracing;
 #define ESCAPE		CTRL('[')
 #define BLANK		' '	/* this is the background character */
 
+static int max_colors;		/* the actual number of colors we'll use */
+static int max_pairs;		/* ...and the number of color pairs */
+
 /* The behavior of mvhline, mvvline for negative/zero length is unspecified,
  * though we can rely on negative x/y values to stop the macro.
  */
@@ -198,7 +201,7 @@ mouse_decode(MEVENT const *ep)
     static char buf[80];
 
     (void) sprintf(buf, "id %2d  at (%2d, %2d, %2d) state %4lx = {",
-	ep->id, ep->x, ep->y, ep->z, ep->bstate);
+		   ep->id, ep->x, ep->y, ep->z, ep->bstate);
 
 #define SHOW(m, s) if ((ep->bstate & m)==m) {strcat(buf,s); strcat(buf, ", ");}
     SHOW(BUTTON1_RELEASED, "release-1");
@@ -246,15 +249,156 @@ mouse_decode(MEVENT const *ep)
  ****************************************************************************/
 
 static void
-getch_test(void)
-/* test the keypad feature */
+wgetch_help(WINDOW *win, bool is_keypad)
+{
+    static const char *help[] =
+    {
+	"g -- triggers a getstr test"
+	,"k -- toggle keypad/literal mode"
+	,"s -- shell out\n"
+	,"q -- quit (x also exits)"
+	,"w -- create a new window"
+#ifdef SIGTSTP
+	,"z -- suspend this process"
+#endif
+    };
+    int y, x;
+    unsigned n;
+
+    getyx(win, y, x);
+    move(0, 0);
+    printw("Type any key to see its %s value.  Also:\n",
+	   is_keypad ? "keypad" : "literal");
+    for (n = 0; n < SIZEOF(help); ++n) {
+	mvprintw(
+		    1 + n % (SIZEOF(help) / 2),
+		    (n >= SIZEOF(help) / 2) ? COLS / 2 : 0,
+		    "%s", help[n]);
+	clrtoeol();
+    }
+    wrefresh(stdscr);
+    wmove(win, y, x);
+}
+
+static void
+wgetch_test(WINDOW *win, int delay)
 {
     char buf[BUFSIZ];
+    int first_y, first_x;
+    int last_y = getmaxy(win) - 1;
     int c;
-    int incount = 0, firsttime = 0;
-    bool blocking = TRUE;
+    int incount = 0;
+    bool blocking = (delay < 0);
+    bool is_keypad = (win == stdscr);
     int y, x;
 
+    keypad(win, is_keypad);	/* should be redundant, but for testing */
+    wtimeout(win, delay);
+    getyx(win, first_y, first_x);
+
+    wgetch_help(win, is_keypad);
+    wsetscrreg(win, first_y, last_y);
+    scrollok(win, TRUE);
+
+    for (;;) {
+	while ((c = wGetchar(win)) == ERR) {
+	    incount++;
+	    if (blocking) {
+		(void) wprintw(win, "%05d: input error\n", incount);
+		break;
+	    } else {
+		(void) wprintw(win, "%05d: input timed out\n", incount);
+	    }
+	}
+	if (c == 'x' || c == 'q' || (c == ERR && blocking)) {
+	    break;
+	} else if (c == 'g') {
+	    waddstr(win, "getstr test: ");
+	    echo();
+	    wgetnstr(win, buf, sizeof(buf) - 1);
+	    noecho();
+	    wprintw(win, "I saw %d characters:\n\t`%s'.\n", strlen(buf), buf);
+	} else if (c == 'k') {
+	    is_keypad = !is_keypad;
+	    keypad(win, is_keypad);
+	    wgetch_help(win, is_keypad);
+	} else if (c == 's') {
+	    ShellOut(TRUE);
+	} else if (c == 'w') {
+	    int high = last_y - first_y + 1;
+	    int wide = getmaxx(win) - first_x;
+	    int old_y, old_x;
+	    int new_y = first_y + getbegy(win);
+	    int new_x = first_x + getbegx(win);
+
+	    getyx(win, old_y, old_x);
+	    if (high > 2 && wide > 2) {
+		WINDOW *wb = newwin(high, wide, new_y, new_x);
+		WINDOW *wi = newwin(high - 2, wide - 2, new_y + 1, new_x + 1);
+
+		box(wb, 0, 0);
+		wrefresh(wb);
+		wmove(wi, 0, 0);
+		wgetch_test(wi, delay);
+		delwin(wi);
+		delwin(wb);
+
+		wgetch_help(win, is_keypad);
+		wmove(win, old_y, old_x);
+		touchwin(win);
+		wrefresh(win);
+	    }
+#ifdef SIGTSTP
+	} else if (c == 'z') {
+	    kill(getpid(), SIGTSTP);
+#endif
+	} else {
+	    wprintw(win, "Key pressed: %04o ", c);
+#ifdef NCURSES_MOUSE_VERSION
+	    if (c == KEY_MOUSE) {
+		MEVENT event;
+
+		getmouse(&event);
+		wprintw(win, "KEY_MOUSE, %s\n", mouse_decode(&event));
+		getyx(win, y, x);
+		move(event.y, event.x);
+		addch('*');
+		wmove(win, y, x);
+	    } else
+#endif /* NCURSES_MOUSE_VERSION */
+	    if (c >= KEY_MIN) {
+		(void) waddstr(win, keyname(c));
+		waddch(win, '\n');
+	    } else if (c > 0x80) {
+		int c2 = (c & 0x7f);
+		if (isprint(c2))
+		    (void) wprintw(win, "M-%c", c2);
+		else
+		    (void) wprintw(win, "M-%s", unctrl(c2));
+		waddstr(win, " (high-half character)\n");
+	    } else {
+		if (isprint(c))
+		    (void) wprintw(win, "%c (ASCII printable character)\n", c);
+		else
+		    (void) wprintw(win, "%s (ASCII control character)\n",
+				   unctrl(c));
+	    }
+	    y = getcury(win);
+	    if (y >= last_y)
+		wmove(win, first_y, 0);
+	    wclrtoeol(win);
+	}
+    }
+
+    wtimeout(win, -1);
+}
+
+static void
+getch_test(void)
+{
+    char buf[BUFSIZ];
+    int delay;
+
     refresh();
 
 #ifdef NCURSES_MOUSE_VERSION
@@ -263,83 +407,22 @@ getch_test(void)
 
     (void) printw("Delay in 10ths of a second (<CR> for blocking input)? ");
     echo();
-    getstr(buf);
+    getnstr(buf, sizeof(buf) - 1);
     noecho();
     nonl();
 
-    if (isdigit(buf[0])) {
-	timeout(atoi(buf) * 100);
-	blocking = FALSE;
+    if (isdigit(UChar(buf[0]))) {
+	delay = atoi(buf) * 100;
+    } else {
+	delay = -1;
     }
-
-    c = '?';
     raw();
-    for (;;) {
-	if (firsttime++) {
-	    printw("Key pressed: %04o ", c);
-#ifdef NCURSES_MOUSE_VERSION
-	    if (c == KEY_MOUSE) {
-		MEVENT event;
-
-		getmouse(&event);
-		printw("KEY_MOUSE, %s\n", mouse_decode(&event));
-	    } else
-#endif /* NCURSES_MOUSE_VERSION */
-	    if (c >= KEY_MIN) {
-		(void) addstr(keyname(c));
-		addch('\n');
-	    } else if (c > 0x80) {
-		int c2 = (c & 0x7f);
-		if (isprint(c2))
-		    (void) printw("M-%c", c2);
-		else
-		    (void) printw("M-%s", unctrl(c2));
-		addstr(" (high-half character)\n");
-	    } else {
-		if (isprint(c))
-		    (void) printw("%c (ASCII printable character)\n", c);
-		else
-		    (void) printw("%s (ASCII control character)\n", unctrl(c));
-	    }
-	    getyx(stdscr, y, x);
-	    if (y >= LINES - 1)
-		move(0, 0);
-	    clrtoeol();
-	}
-
-	if (c == 'g') {
-	    addstr("getstr test: ");
-	    echo();
-	    getstr(buf);
-	    noecho();
-	    printw("I saw `%s'.\n", buf);
-	}
-	if (c == 's') {
-	    ShellOut(TRUE);
-	}
-	if (c == 'x' || c == 'q' || (c == ERR && blocking))
-	    break;
-	if (c == '?') {
-	    addstr("Type any key to see its keypad value.  Also:\n");
-	    addstr("g -- triggers a getstr test\n");
-	    addstr("s -- shell out\n");
-	    addstr("q -- quit\n");
-	    addstr("? -- repeats this help message\n");
-	}
-
-	while ((c = Getchar()) == ERR)
-	    if (!blocking)
-		(void) printw("%05d: input timed out\n", incount++);
-	    else {
-		(void) printw("%05d: input error\n", incount++);
-		break;
-	    }
-    }
+    move(5, 0);
+    wgetch_test(stdscr, delay);
 
 #ifdef NCURSES_MOUSE_VERSION
     mousemask(0, (mmask_t *) 0);
 #endif
-    timeout(-1);
     erase();
     noraw();
     nl();
@@ -347,21 +430,43 @@ getch_test(void)
 }
 
 static int
-show_attr(int row, int skip, chtype attr, const char *name, bool once)
+show_attr(int row, int skip, chtype attr, const char *name)
 {
+    static const char *string = "abcde fghij klmno pqrst uvwxy z";
     int ncv = tigetnum("ncv");
 
     mvprintw(row, 8, "%s mode:", name);
     mvprintw(row, 24, "|");
     if (skip)
 	printw("%*s", skip, " ");
-    if (once)
-	attron(attr);
-    else
-	attrset(attr);
-    addstr("abcde fghij klmno pqrst uvwxy z");
-    if (once)
-	attroff(attr);
+    attrset(attr);
+    /*
+     * If we're to write a string in the alternate character set, it is not
+     * sufficient to just set A_ALTCHARSET.  We have to perform the mapping
+     * that corresponds.  This is not needed for vt100-compatible devices
+     * because the acs_map[] is 1:1, but for PC-style devices such as Linux
+     * console, the acs_map[] is scattered about the range.
+     *
+     * The addch/addstr functions do not themselves do this mapping, since it
+     * is possible to turn off the A_ALTCHARSET flag for the characters which
+     * are added, and it would be an unexpected result to have the mapped
+     * characters visible on the screen.
+     *
+     * This example works because the indices into acs_map[] are mostly from
+     * the lowercase characters.
+     */
+    if (attr & A_ALTCHARSET) {
+	const char *s = string;
+	while (*s) {
+	    int ch = *s++;
+	    if ((ch = acs_map[ch]) == 0)
+		ch = ' ';
+	    addch(ch);
+	}
+    } else {
+	addstr(string);
+    }
+    attroff(attr);
     if (skip)
 	printw("%*s", skip, " ");
     printw("|");
@@ -383,7 +488,7 @@ show_attr(int row, int skip, chtype attr, const char *name, bool once)
 	    };
 	    unsigned n;
 	    bool found = FALSE;
-	    for (n = 0; n < sizeof(table) / sizeof(table[0]); n++) {
+	    for (n = 0; n < SIZEOF(table); n++) {
 		if ((table[n] & attr) != 0
 		    && ((1 << n) & ncv) != 0) {
 		    found = TRUE;
@@ -398,19 +503,23 @@ show_attr(int row, int skip, chtype attr, const char *name, bool once)
 }
 
 static bool
-attr_getc(int *skip, int *fg, int *bg)
+attr_getc(int *skip, int *fg, int *bg, int *ac)
 {
     int ch = Getchar();
 
     if (isdigit(ch)) {
 	*skip = (ch - '0');
-	return TRUE;
     } else if (ch == CTRL('L')) {
 	touchwin(stdscr);
 	touchwin(curscr);
-	return TRUE;
     } else if (has_colors()) {
 	switch (ch) {
+	case 'a':
+	    *ac = 0;
+	    break;
+	case 'A':
+	    *ac = A_ALTCHARSET;
+	    break;
 	case 'f':
 	    *fg = (*fg + 1);
 	    break;
@@ -426,17 +535,27 @@ attr_getc(int *skip, int *fg, int *bg)
 	default:
 	    return FALSE;
 	}
-	if (*fg >= COLORS)
+	if (*fg >= max_colors)
 	    *fg = 0;
 	if (*fg < 0)
-	    *fg = COLORS - 1;
-	if (*bg >= COLORS)
+	    *fg = max_colors - 1;
+	if (*bg >= max_colors)
 	    *bg = 0;
 	if (*bg < 0)
-	    *bg = COLORS - 1;
-	return TRUE;
+	    *bg = max_colors - 1;
+    } else {
+	switch (ch) {
+	case 'a':
+	    *ac = 0;
+	    break;
+	case 'A':
+	    *ac = A_ALTCHARSET;
+	    break;
+	default:
+	    return FALSE;
+	}
     }
-    return FALSE;
+    return TRUE;
 }
 
 static void
@@ -447,7 +566,8 @@ attr_test(void)
     int skip = tigetnum("xmc");
     int fg = COLOR_BLACK;	/* color pair 0 is special */
     int bg = COLOR_BLACK;
-    bool *pairs = (bool *) calloc(COLOR_PAIRS, sizeof(bool));
+    int ac = 0;
+    bool *pairs = (bool *) calloc(max_pairs, sizeof(bool));
     pairs[0] = TRUE;
 
     if (skip < 0)
@@ -460,7 +580,7 @@ attr_test(void)
 	int normal = A_NORMAL | BLANK;
 
 	if (has_colors()) {
-	    int pair = (fg * COLORS) + bg;
+	    int pair = (fg * max_colors) + bg;
 	    if (!pairs[pair]) {
 		init_pair(pair, fg, bg);
 		pairs[pair] = TRUE;
@@ -472,28 +592,31 @@ attr_test(void)
 
 	mvaddstr(0, 20, "Character attribute test display");
 
-	row = show_attr(row, n, A_STANDOUT, "STANDOUT", TRUE);
-	row = show_attr(row, n, A_REVERSE, "REVERSE", TRUE);
-	row = show_attr(row, n, A_BOLD, "BOLD", TRUE);
-	row = show_attr(row, n, A_UNDERLINE, "UNDERLINE", TRUE);
-	row = show_attr(row, n, A_DIM, "DIM", TRUE);
-	row = show_attr(row, n, A_BLINK, "BLINK", TRUE);
-	row = show_attr(row, n, A_PROTECT, "PROTECT", TRUE);
-	row = show_attr(row, n, A_INVIS, "INVISIBLE", TRUE);
-	row = show_attr(row, n, A_NORMAL, "NORMAL", FALSE);
+	row = show_attr(row, n, ac | A_STANDOUT, "STANDOUT");
+	row = show_attr(row, n, ac | A_REVERSE, "REVERSE");
+	row = show_attr(row, n, ac | A_BOLD, "BOLD");
+	row = show_attr(row, n, ac | A_UNDERLINE, "UNDERLINE");
+	row = show_attr(row, n, ac | A_DIM, "DIM");
+	row = show_attr(row, n, ac | A_BLINK, "BLINK");
+	row = show_attr(row, n, ac | A_PROTECT, "PROTECT");
+	row = show_attr(row, n, ac | A_INVIS, "INVISIBLE");
+	row = show_attr(row, n, ac | A_NORMAL, "NORMAL");
 
 	mvprintw(row, 8,
-	    "This terminal does %shave the magic-cookie glitch",
-	    tigetnum("xmc") > -1 ? "" : "not ");
+		 "This terminal does %shave the magic-cookie glitch",
+		 tigetnum("xmc") > -1 ? "" : "not ");
 	mvprintw(row + 1, 8,
-	    "Enter a digit to set gaps on each side of displayed attributes");
+		 "Enter a digit to set gaps on each side of displayed attributes");
 	mvprintw(row + 2, 8,
-	    "^L = repaint");
+		 "^L = repaint");
 	if (has_colors())
-	    printw(".  f/F/b/F toggle colors (now %d/%d)", fg, bg);
+	    printw(".  f/F/b/F toggle colors (now %d/%d), a/A altcharset (%d)",
+		   fg, bg, ac != 0);
+	else
+	    printw(".  a/A altcharset (%d)", ac != 0);
 
 	refresh();
-    } while (attr_getc(&n, &fg, &bg));
+    } while (attr_getc(&n, &fg, &bg, &ac));
 
     free((char *) pairs);
     bkgdset(A_NORMAL | BLANK);
@@ -530,7 +653,7 @@ static NCURSES_CONST char *color_names[] =
 static void
 show_color_name(int y, int x, int color)
 {
-    if (COLORS > 8)
+    if (max_colors > 8)
 	mvprintw(y, x, "%02d   ", color);
     else
 	mvaddstr(y, x, color_names[color]);
@@ -547,29 +670,30 @@ color_test(void)
     refresh();
     (void) printw("There are %d color pairs\n", COLOR_PAIRS);
 
-    width = (COLORS > 8) ? 4 : 8;
-    hello = (COLORS > 8) ? "Test" : "Hello";
+    width = (max_colors > 8) ? 4 : 8;
+    hello = (max_colors > 8) ? "Test" : "Hello";
 
     for (base = 0; base < 2; base++) {
-	top = (COLORS > 8) ? 0 : base * (COLORS + 3);
+	top = (max_colors > 8) ? 0 : base * (max_colors + 3);
 	clrtobot();
 	(void) mvprintw(top + 1, 0,
-	    "%dx%d matrix of foreground/background colors, bright *%s*\n",
-	    COLORS, COLORS,
-	    base ? "on" : "off");
-	for (i = 0; i < COLORS; i++)
+			"%dx%d matrix of foreground/background colors, bright *%s*\n",
+			max_colors, max_colors,
+			base ? "on" : "off");
+	for (i = 0; i < max_colors; i++)
 	    show_color_name(top + 2, (i + 1) * width, i);
-	for (i = 0; i < COLORS; i++)
+	for (i = 0; i < max_colors; i++)
 	    show_color_name(top + 3 + i, 0, i);
-	for (i = 1; i < COLOR_PAIRS; i++) {
-	    init_pair(i, i % COLORS, i / COLORS);
+	for (i = 1; i < max_pairs; i++) {
+	    init_pair(i, i % max_colors, i / max_colors);
 	    attron((attr_t) COLOR_PAIR(i));
 	    if (base)
 		attron((attr_t) A_BOLD);
-	    mvaddstr(top + 3 + (i / COLORS), (i % COLORS + 1) * width, hello);
+	    mvaddstr(top + 3 + (i / max_colors), (i % max_colors + 1) *
+		     width, hello);
 	    attrset(A_NORMAL);
 	}
-	if ((COLORS > 8) || base)
+	if ((max_colors > 8) || base)
 	    Pause();
     }
 
@@ -609,7 +733,6 @@ color_edit(void)
 {
     int i, this_c = 0, value = 0, current = 0, field = 0;
     int last_c;
-    int max_colors = COLORS > 16 ? 16 : COLORS;
 
     refresh();
 
@@ -627,9 +750,9 @@ color_edit(void)
 
 	for (i = 0; i < max_colors; i++) {
 	    mvprintw(2 + i, 0, "%c %-8s:",
-		(i == current ? '>' : ' '),
-		(i < (int) SIZEOF(color_names)
-		    ? color_names[i] : ""));
+		     (i == current ? '>' : ' '),
+		     (i < (int) SIZEOF(color_names)
+		      ? color_names[i] : ""));
 	    attrset(COLOR_PAIR(i));
 	    addstr("        ");
 	    attrset(A_NORMAL);
@@ -666,9 +789,9 @@ color_edit(void)
 	}
 
 	mvaddstr(max_colors + 3, 0,
-	    "Use up/down to select a color, left/right to change fields.");
+		 "Use up/down to select a color, left/right to change fields.");
 	mvaddstr(max_colors + 4, 0,
-	    "Modify field by typing nnn=, nnn-, or nnn+.  ? for help.");
+		 "Modify field by typing nnn=, nnn-, or nnn+.  ? for help.");
 
 	move(2 + current, 0);
 
@@ -882,7 +1005,7 @@ show_upper_chars(int first)
     erase();
     attron(A_BOLD);
     mvprintw(0, 20, "Display of %s Character Codes %d to %d",
-	C1 ? "C1" : "GR", first, last);
+	     C1 ? "C1" : "GR", first, last);
     attroff(A_BOLD);
     refresh();
 
@@ -983,9 +1106,9 @@ acs_display(void)
 	    break;
 	}
 	mvprintw(LINES - 3, 0,
-	    "Note: ANSI terminals may not display C1 characters.");
+		 "Note: ANSI terminals may not display C1 characters.");
 	mvprintw(LINES - 2, 0,
-	    "Select: a=ACS, 0=C1, 1,2,3=GR characters, q=quit");
+		 "Select: a=ACS, 0=C1, 1,2,3=GR characters, q=quit");
 	refresh();
     } while ((c = Getchar()) != 'x' && c != 'q');
 
@@ -1064,7 +1187,7 @@ test_sgr_attributes(void)
 
 	bkgdset(normal);
 	mvprintw(LINES - 2, 1, "%s background. ", pass == 0 ? "Dark" :
-	    "Light");
+		 "Light");
 	clrtoeol();
 	Pause();
     }
@@ -1146,7 +1269,7 @@ newwin_legend(FRAME * curp)
 	{
 	    "^R = restore window", 0
 	},
-#ifdef HAVE_WRESIZE
+#if HAVE_WRESIZE
 	{
 	    "^X = resize", 0
 	},
@@ -1156,7 +1279,7 @@ newwin_legend(FRAME * curp)
 	}
     };
     size_t n;
-    int y, x;
+    int x;
     bool do_keypad = HaveKeypad(curp);
     bool do_scroll = HaveScroll(curp);
     char buf[BUFSIZ];
@@ -1177,7 +1300,7 @@ newwin_legend(FRAME * curp)
 	    sprintf(buf, legend[n].msg, do_keypad ? "/ESC" : "");
 	    break;
 	}
-	getyx(stdscr, y, x);
+	x = getcurx(stdscr);
 	addstr((COLS < (x + 3 + (int) strlen(buf))) ? "\n" : (n ? ", " : ""));
 	addstr(buf);
     }
@@ -1196,8 +1319,8 @@ transient(FRAME * curp, NCURSES_CONST char *msg)
 
     move(LINES - 1, 0);
     printw("%s characters are echoed, window should %sscroll.",
-	HaveKeypad(curp) ? "Non-arrow" : "All other",
-	HaveScroll(curp) ? "" : "not ");
+	   HaveKeypad(curp) ? "Non-arrow" : "All other",
+	   HaveScroll(curp) ? "" : "not ");
     clrtoeol();
 }
 
@@ -1477,7 +1600,7 @@ acs_and_scroll(void)
 	    }
 	    break;
 
-#ifdef HAVE_WRESIZE
+#if HAVE_WRESIZE
 	case CTRL('X'):	/* resize window */
 	    if (current) {
 		pair *tmp, ul, lr;
@@ -1585,8 +1708,8 @@ acs_and_scroll(void)
 	wrefresh(usescr);
     } while
 	((c = wGetchar(usescr)) != QUIT
-	&& !((c == ESCAPE) && (usescr->_use_keypad))
-	&& (c != ERR));
+	 && !((c == ESCAPE) && (usescr->_use_keypad))
+	 && (c != ERR));
 
   breakout:
     while (current != 0)
@@ -1739,38 +1862,38 @@ demo_panels(void)
 	PANEL *p5;
 
 	p1 = mkpanel(COLOR_RED,
-	    LINES / 2 - 2,
-	    COLS / 8 + 1,
-	    0,
-	    0);
+		     LINES / 2 - 2,
+		     COLS / 8 + 1,
+		     0,
+		     0);
 	set_panel_userptr(p1, "p1");
 
 	p2 = mkpanel(COLOR_GREEN,
-	    LINES / 2 + 1,
-	    COLS / 7,
-	    LINES / 4,
-	    COLS / 10);
+		     LINES / 2 + 1,
+		     COLS / 7,
+		     LINES / 4,
+		     COLS / 10);
 	set_panel_userptr(p2, "p2");
 
 	p3 = mkpanel(COLOR_YELLOW,
-	    LINES / 4,
-	    COLS / 10,
-	    LINES / 2,
-	    COLS / 9);
+		     LINES / 4,
+		     COLS / 10,
+		     LINES / 2,
+		     COLS / 9);
 	set_panel_userptr(p3, "p3");
 
 	p4 = mkpanel(COLOR_BLUE,
-	    LINES / 2 - 2,
-	    COLS / 8,
-	    LINES / 2 - 2,
-	    COLS / 3);
+		     LINES / 2 - 2,
+		     COLS / 8,
+		     LINES / 2 - 2,
+		     COLS / 3);
 	set_panel_userptr(p4, "p4");
 
 	p5 = mkpanel(COLOR_MAGENTA,
-	    LINES / 2 - 2,
-	    COLS / 8,
-	    LINES / 2,
-	    COLS / 2 - 2);
+		     LINES / 2 - 2,
+		     COLS / 8,
+		     LINES / 2,
+		     COLS / 2 - 2);
 	set_panel_userptr(p5, "p5");
 
 	fill_panel(p1);
@@ -1943,6 +2066,7 @@ demo_panels(void)
 
 #define GRIDSIZE	3
 
+static bool pending_pan = FALSE;
 static bool show_panner_legend = TRUE;
 
 static int
@@ -1950,10 +2074,10 @@ panner_legend(int line)
 {
     static const char *const legend[] =
     {
-	"Use arrow keys (or U,D,L,R) to pan, q to quit (?,t,s flags)",
-	"Use ! to shell-out.  Toggle legend:?, timer:t, scroll mark:s.",
+	"Use arrow keys (or U,D,L,R) to pan, q to quit, ! to shell-out.",
 	"Use +,- (or j,k) to grow/shrink the panner vertically.",
-	"Use <,> (or h,l) to grow/shrink the panner horizontally."
+	"Use <,> (or h,l) to grow/shrink the panner horizontally.",
+	"Number repeats.  Toggle legend:?, timer:t, scroll mark:s."
     };
     int n = (SIZEOF(legend) - (LINES - line));
     if (line < LINES && (n >= 0)) {
@@ -1982,8 +2106,8 @@ panner_v_cleanup(int from_y, int from_x, int to_y)
 
 static void
 panner(WINDOW *pad,
-    int top_x, int top_y, int porty, int portx,
-    int (*pgetc) (WINDOW *))
+       int top_x, int top_y, int porty, int portx,
+       int (*pgetc) (WINDOW *))
 {
 #if HAVE_GETTIMEOFDAY
     struct timeval before, after;
@@ -2205,29 +2329,31 @@ panner(WINDOW *pad,
 	mvaddch(porty - 1, top_x - 1, ACS_LLCORNER);
 	mvaddch(porty - 1, portx - 1, ACS_LRCORNER);
 
+	if (!pending_pan) {
 #if HAVE_GETTIMEOFDAY
-	gettimeofday(&before, 0);
+	    gettimeofday(&before, 0);
 #endif
-	wnoutrefresh(stdscr);
+	    wnoutrefresh(stdscr);
 
-	pnoutrefresh(pad,
-	    basey, basex,
-	    top_y, top_x,
-	    porty - (pxmax > portx) - 1,
-	    portx - (pymax > porty) - 1);
+	    pnoutrefresh(pad,
+			 basey, basex,
+			 top_y, top_x,
+			 porty - (pxmax > portx) - 1,
+			 portx - (pymax > porty) - 1);
 
-	doupdate();
+	    doupdate();
 #if HAVE_GETTIMEOFDAY
-	if (timing) {
-	    double elapsed;
-	    gettimeofday(&after, 0);
-	    elapsed = (after.tv_sec + after.tv_usec / 1.0e6)
-		- (before.tv_sec + before.tv_usec / 1.0e6);
-	    move(LINES - 1, COLS - 20);
-	    printw("Secs: %2.03f", elapsed);
-	    refresh();
+	    if (timing) {
+		double elapsed;
+		gettimeofday(&after, 0);
+		elapsed = (after.tv_sec + after.tv_usec / 1.0e6)
+		    - (before.tv_sec + before.tv_usec / 1.0e6);
+		move(LINES - 1, COLS - 20);
+		printw("Secs: %2.03f", elapsed);
+		refresh();
+	    }
+#endif
 	}
-#endif
 
     } while
 	((c = pgetc(pad)) != KEY_EXIT);
@@ -2235,44 +2361,73 @@ panner(WINDOW *pad,
     scrollok(stdscr, TRUE);	/* reset to driver's default */
 }
 
-static
-int
+static int
 padgetch(WINDOW *win)
 {
+    static int count;
+    static int last;
     int c;
 
-    switch (c = wGetchar(win)) {
-    case '!':
-	ShellOut(FALSE);
-	return KEY_REFRESH;
-    case CTRL('r'):
-	endwin();
-	refresh();
-	return KEY_REFRESH;
-    case CTRL('l'):
-	return KEY_REFRESH;
-    case 'U':
-	return (KEY_UP);
-    case 'D':
-	return (KEY_DOWN);
-    case 'R':
-	return (KEY_RIGHT);
-    case 'L':
-	return (KEY_LEFT);
-    case '+':
-	return (KEY_IL);
-    case '-':
-	return (KEY_DL);
-    case '>':
-	return (KEY_IC);
-    case '<':
-	return (KEY_DC);
-    case ERR:			/* FALLTHRU */
-    case 'q':
-	return (KEY_EXIT);
-    default:
-	return (c);
+    if ((pending_pan = (count > 0)) != FALSE) {
+	count--;
+	pending_pan = (count != 0);
+    } else {
+	for (;;) {
+	    switch (c = wGetchar(win)) {
+	    case '!':
+		ShellOut(FALSE);
+		/* FALLTHRU */
+	    case CTRL('r'):
+		endwin();
+		refresh();
+		c = KEY_REFRESH;
+		break;
+	    case CTRL('l'):
+		c = KEY_REFRESH;
+		break;
+	    case 'U':
+		c = KEY_UP;
+		break;
+	    case 'D':
+		c = KEY_DOWN;
+		break;
+	    case 'R':
+		c = KEY_RIGHT;
+		break;
+	    case 'L':
+		c = KEY_LEFT;
+		break;
+	    case '+':
+		c = KEY_IL;
+		break;
+	    case '-':
+		c = KEY_DL;
+		break;
+	    case '>':
+		c = KEY_IC;
+		break;
+	    case '<':
+		c = KEY_DC;
+		break;
+	    case ERR:		/* FALLTHRU */
+	    case 'q':
+		count = 0;
+		c = KEY_EXIT;
+		break;
+	    default:
+		if (c >= '0' && c <= '9') {
+		    count = count * 10 + (c - '0');
+		    continue;
+		}
+		break;
+	    }
+	    last = c;
+	    break;
+	}
+	if (count > 0)
+	    count--;
     }
+    return (last);
 }
 
 #define PAD_HIGH 200
@@ -2379,7 +2534,7 @@ flushinp_test(WINDOW *win)
 
     mvwaddstr(win, 2, 1, "Type random keys for 5 seconds.");
     mvwaddstr(win, 3, 1,
-	"These should be discarded (not echoed) after the subwindow goes away.");
+	      "These should be discarded (not echoed) after the subwindow goes away.");
     wrefresh(win);
 
     for (i = 0; i < 5; i++) {
@@ -2396,11 +2551,11 @@ flushinp_test(WINDOW *win)
     napms(1000);
 
     mvwaddstr(win, 2, 1,
-	"If you were still typing when the window timer expired,");
+	      "If you were still typing when the window timer expired,");
     mvwaddstr(win, 3, 1,
-	"or else you typed nothing at all while it was running,");
+	      "or else you typed nothing at all while it was running,");
     mvwaddstr(win, 4, 1,
-	"test was invalid.  You'll see garbage or nothing at all. ");
+	      "test was invalid.  You'll see garbage or nothing at all. ");
     mvwaddstr(win, 6, 1, "Press a key");
     wmove(win, 9, 10);
     wrefresh(win);
@@ -2408,7 +2563,7 @@ flushinp_test(WINDOW *win)
     wGetchar(win);
     flushinp();
     mvwaddstr(win, 12, 0,
-	"If you see any key other than what you typed, flushinp() is broken.");
+	      "If you see any key other than what you typed, flushinp() is broken.");
     Continue(win);
 
     wmove(win, 9, 10);
@@ -2417,7 +2572,7 @@ flushinp_test(WINDOW *win)
     wmove(win, 12, 0);
     clrtoeol();
     waddstr(win,
-	"What you typed should now have been deleted; if not, wdelch() failed.");
+	    "What you typed should now have been deleted; if not, wdelch() failed.");
     Continue(win);
 
     cbreak();
@@ -2483,7 +2638,7 @@ menu_test(void)
     mvaddstr(2, 0, "  Use up and down arrow to move the select bar.");
     mvaddstr(3, 0, "  'n' and 'p' act like arrows.");
     mvaddstr(4, 0,
-	"  'b' and 'f' scroll up/down (page), 'u' and 'd' (line).");
+	     "  'b' and 'f' scroll up/down (page), 'u' and 'd' (line).");
     mvaddstr(5, 0, "  Press return to exit.");
     refresh();
 
@@ -2512,7 +2667,7 @@ menu_test(void)
     }
 
     (void) mvprintw(LINES - 2, 0,
-	"You chose: %s\n", item_name(current_item(m)));
+		    "You chose: %s\n", item_name(current_item(m)));
     (void) addstr("Press any key to continue...");
     wGetchar(stdscr);
 
@@ -2548,6 +2703,7 @@ static struct {
 	T_TBL(TRACE_ICALLS),
 	T_TBL(TRACE_CCALLS),
 	T_TBL(TRACE_DATABASE),
+	T_TBL(TRACE_ATTRS),
 	T_TBL(TRACE_MAXIMUM),
     {
 	(char *) 0, 0
@@ -2677,7 +2833,7 @@ trace_set(void)
     _tracef("trace level interactively set to %s", tracetrace(_nc_tracing));
 
     (void) mvprintw(LINES - 2, 0,
-	"Trace level is %s\n", tracetrace(_nc_tracing));
+		    "Trace level is %s\n", tracetrace(_nc_tracing));
     (void) addstr("Press any key to continue...");
     wGetchar(stdscr);
 
@@ -2945,7 +3101,7 @@ form_virtualize(FORM * f, WINDOW *w)
 	    mode = REQ_INS_MODE;
 	c = mode;
     } else {
-	for (n = 0; n < sizeof(lookup) / sizeof(lookup[0]); n++) {
+	for (n = 0; n < SIZEOF(lookup); n++) {
 	    if (lookup[n].code == c) {
 		c = lookup[n].result;
 		break;
@@ -3289,9 +3445,9 @@ usage(void)
 #endif
     };
     size_t n;
-    for (n = 0; n < sizeof(tbl) / sizeof(tbl[0]); n++)
+    for (n = 0; n < SIZEOF(tbl); n++)
 	fprintf(stderr, "%s\n", tbl[n]);
-    exit(EXIT_FAILURE);
+    ExitProgram(EXIT_FAILURE);
 }
 
 static void
@@ -3427,12 +3583,22 @@ main(int argc, char *argv[])
     /* tests, in general, will want these modes */
     if (has_colors()) {
 	start_color();
-#ifdef NCURSES_VERSION
+#ifdef NCURSES_VERSION_PATCH
+	max_colors = COLORS > 16 ? 16 : COLORS;
+#if HAVE_USE_DEFAULT_COLORS
 	if (default_colors)
 	    use_default_colors();
+#if NCURSES_VERSION_PATCH >= 20000708
 	else if (assumed_colors)
 	    assume_default_colors(default_fg, default_bg);
 #endif
+#endif
+#else /* normal SVr4 curses */
+	max_colors = COLORS > 8 ? 8 : COLORS;
+#endif
+	max_pairs = (max_colors * max_colors);
+	if (max_pairs < COLOR_PAIRS)
+	    max_pairs = COLOR_PAIRS;
     }
     set_terminal_modes();
     def_prog_mode();
@@ -3447,9 +3613,9 @@ main(int argc, char *argv[])
     (void) printf("Welcome to %s.  Press ? for help.\n", curses_version());
 #elif defined(NCURSES_VERSION_MAJOR) && defined(NCURSES_VERSION_MINOR) && defined(NCURSES_VERSION_PATCH)
     (void) printf("Welcome to ncurses %d.%d.%d.  Press ? for help.\n",
-	NCURSES_VERSION_MAJOR,
-	NCURSES_VERSION_MINOR,
-	NCURSES_VERSION_PATCH);
+		  NCURSES_VERSION_MAJOR,
+		  NCURSES_VERSION_MINOR,
+		  NCURSES_VERSION_PATCH);
 #else
     (void) puts("Welcome to ncurses.  Press ? for help.");
 #endif
@@ -3497,7 +3663,7 @@ main(int argc, char *argv[])
 		if (command == 0)
 		    command = 'q';
 		break;
-	    } else if (command == 0 && !isspace(ch)) {
+	    } else if (command == 0 && !isspace(UChar(ch))) {
 		command = ch;
 	    } else if (ch == '\n' || ch == '\r') {
 		if (command != 0)
