@@ -29,7 +29,7 @@
 /****************************************************************************
  *  Author: Thomas E. Dickey <dickey@clark.net> 1996                        *
  ****************************************************************************/
-/* $Id: test.priv.h,v 1.19 2000/09/02 19:31:58 tom Exp $ */
+/* $Id: test.priv.h,v 1.25 2001/09/15 21:25:18 tom Exp $ */
 
 #if HAVE_CONFIG_H
 #include <ncurses_cfg.h>
@@ -87,17 +87,19 @@ extern int optind;
 #define GCC_UNUSED /* nothing */
 #endif
 
+#define UChar(c)    ((unsigned char)(c))
+
 #define SIZEOF(table)	(sizeof(table)/sizeof(table[0]))
 
 #if defined(NCURSES_VERSION) && HAVE_NC_ALLOC_H
 #include <nc_alloc.h>
 #else
-#define typeMalloc(type,n) (type *) malloc(n * sizeof(type))
-#define typeRealloc(type,n,p) (type *) realloc(p, n * sizeof(type))
+#define typeMalloc(type,n) (type *) malloc((n) * sizeof(type))
+#define typeRealloc(type,n,p) (type *) realloc(p, (n) * sizeof(type))
 #endif
 
 #ifndef ExitProgram
-#define ExitProgram(code) return code
+#define ExitProgram(code) exit(code)
 #endif
 
 #ifndef EXIT_SUCCESS
